<?
/**
 * XML Class
 *
 * @version 0.8.0 2003-06-04 17:30
 * @copyright (C)2003 Jan Marsch
 */

function htmlDecode($string){
	$string = strtr($string, array_flip(get_html_translation_table(HTML_ENTITIES)));
	return preg_replace("/&#([0-9]+);/me", "chr('\\1')", $string);
	}

function htmlEncode($string){
	return str_replace("&", "&amp;", htmlentities($string));
	}

class xmlNode{

	var $nodeName;
	var $nodeType;
	var $nodeValue;
	var $parentNode;
	var $attributes		= array();
	var $childNodes		= array();

	function xmlNode($name, $type, $value){
		$this->nodeName		= $name;
		$this->nodeType		= $type;
		$this->nodeValue	= $value;
		}

	function appendChild($child){
		$this->childNodes[] =& $child;
		$child->parentNode =& $this;
		}

	function getPath(){
		if($this->parentNode) $parent = $this->parentNode->getPath();
		return $parent."/".$this->nodeName;
		}

	function trace($indent = 0){
		if($indent == 0) echo "<pre style=\"font-size:10pt\">";

		if($this->nodeType != 1){
			$val = preg_replace('/&amp;(#?[\w]+;)/', "<i style=\"color:#990000\">&amp;\\1</i>", htmlspecialchars($this->nodeValue));
			echo str_repeat("\t", $indent).str_replace("\n", "\n".str_repeat("\t", $indent), $val)."\n";
			}
		else{
			echo str_repeat("\t", $indent)."<b style=\"color:#990099\">&lt;".$this->nodeName."</b>";

			if(count($this->attributes)){
				foreach($this->attributes as $key=>$val){
					$val = preg_replace('/&(#?[\w]+;)/', "<i style=\"color:#990000\">&amp;\\1</i>", htmlspecialchars($val));
					echo " <i style=\"color:#000099\">$key=\"</i><i style=\"color:#006600\">".$val."</i><i style=\"color:#000099\">\"</i>";
					}
				}

			if(!count($this->childNodes)){
				echo "<b style=\"color:#990099\"> /&gt;</b>\n";
				}
			else{
				echo "<b style=\"color:#990099\">&gt;</b>\n";

				for($i = 0; $i < count($this->childNodes); $i++){
					$this->childNodes[$i]->trace($indent+1);
					}

				echo str_repeat("\t", $indent)."<b style=\"color:#990099\">&lt;/".$this->nodeName."&gt;</b>\n";
				}
			}

		if($indent == 0) echo "</pre>";
		}

	function toString($indent = 0){
		if($this->nodeType != 1){
			$echo .= htmlEncode($this->nodeValue)."\n";
			}
		else{
			$echo .= str_repeat("\t", $indent)."<".$this->nodeName;

			if(count($this->attributes)){
				foreach($this->attributes as $key=>$val){
					$echo .= " $key=\"".htmlEncode($val)."\"";
					}
				}

			if(!count($this->childNodes)){
				$echo .= " />\n";
				}
			else{
				$echo .= ">\n";

				for($i = 0; $i < count($this->childNodes); $i++){
					$echo .= $this->childNodes[$i]->toString($indent+1);
					}

				$echo .= str_repeat("\t", $indent)."</".$this->nodeName.">\n";
				}
			}

		return $echo;
		}
	}

//*************************************
//*************************************
//*************************************

class xml{

	function &file($file, $node = false){

		if(is_object($node)){
			$fp = fopen($file, "w");

			if(!$fp) return false;

			fputs($fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n".$node->toString());
			fclose($fp);

			return true;
			}
		else{
			$fp = fopen($file, "rb");

			if(!$fp) return false;

			while(!feof($fp)){
				$string .= fgets($fp, 512);
				}
			fclose($fp);

			return $this->parse($string);
			}
		}

	function &parse($string){
		$string = utf8_decode($string);
		$parser = xml_parser_create();
		xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, false);
		xml_parser_set_option($parser, XML_OPTION_TARGET_ENCODING, "ISO-8859-1");
		xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, true);
		xml_parse_into_struct($parser, $string, $val, $idx);
		xml_parser_free($parser);

		$level = 0;
		$stack = array();

		$stack[$level] =& $this->createElement("root", null);

		for($i = 0; $i < count($val); $i++){

			switch($val[$i]["type"]){
				case "open":
					$level++;
					$stack[$level] =& $this->createElement($val[$i]["tag"]);
					//****** $stack[$level]->attributes = $val[$i]["attributes"];
					if($val[$i]["attributes"]){
						foreach($val[$i]["attributes"] as $attKey=>$attVal){
							$stack[$level]->attributes[$attKey] = htmlDecode($attVal);
							}
						}

					if($val[$i]["value"]){
						$text =& $this->createTextNode(htmlDecode($val[$i]["value"]));
						$stack[$level]->appendChild(&$text);
						}
					break;

				case "close":
					$stack[$level-1]->appendChild(&$stack[$level]);
					$root =& array_pop($stack);
					$level--;
					break;

				case "complete":
					$level++;
					$stack[$level] =& $this->createElement($val[$i]["tag"]);
					//****** $stack[$level]->attributes = $val[$i]["attributes"];
					if($val[$i]["attributes"]){
						foreach($val[$i]["attributes"] as $attKey=>$attVal){
							$stack[$level]->attributes[$attKey] = htmlDecode($attVal);
							}
						}

					if($val[$i]["value"]){
						$text =& $this->createTextNode(htmlDecode($val[$i]["value"]));
						$stack[$level]->appendChild(&$text);
						}
					$stack[$level-1]->appendChild(&$stack[$level]);
					$root =& array_pop($stack);
					$level--;
					break;

				case "cdata":
					if($val[$i]["value"]){
						$text =& $this->createTextNode(htmlDecode($val[$i]["value"]));
						$stack[$level]->appendChild(&$text);
						}
					break;
				}
			}

		$root->parentNode = null;
		return $root;
		}

	function &createElement($name){
		$element = new xmlNode($name, 1, null);
		return $element;
		}

	function &createTextNode($value){
		if(substr($value, 0, 2) == "<!")	$element = new xmlNode("#comment", 8, $value);
		else								$element = new xmlNode("#text", 3, $value);
		return $element;
		}

	function getNodes(&$node, $query, $level = 0){
		static $res;
		if(!$level) $res = array();

		$path = explode("/", $query);
		array_shift($path);
		preg_match("/^([\w*]*)(@([\w*]+)(='(.*)')?)?$/i", $path[$level], $match);

		if($match[1] != $node->nodeName
		&& $match[1] != "*"
		&& $match[1] != ""){
			return false;
			}

		if($match[3]){
			if($match[3] == "*"){
				if(!count($node->attributes)){
					return false;
					}
				else{
					if($match[4]
					&& !in_array($match[5], array_values($node->attributes))){
						return false;
						}
					}
				}
			else{
				if(!isset($node->attributes[$match[3]])){
					return false;
					}
				else{
					if($match[4]
					&& $node->attributes[$match[3]] != $match[5]){
						return false;
						}
					}
				}
			}

		if(count($path) == $level+1){
			$res[] =& $node;
			if(!$level) return $res;
			return true;
			}

		for($i = 0; $i < count($node->childNodes); $i++){
			$this->getNodes($node->childNodes[$i], $query, $level+1);
			}

		return $res;
		}


	function &getElementsByName($name){
		for($i = 0; $i < count($this->nodes); $i++){
/***
			if($this->nodes[$i]->nodeName == $name
			&& $this->nodes[$i]->parentNode
			&& count($this->nodes[$i]->childNodes)){
				$nodes[] =& $this->nodes[$i];
				}
***/
			if($this->nodes[$i]->nodeName == $name){
				$nodes[] =& $this->nodes[$i];
				}
			}

		return $nodes;
		}
	}
?>