#!/usr/bin/perl -w

# XML Wrapper for Flash, Aaron Steed 2007
#
# Takes a request for a html document and formats it for my 3rd Flash portfolio
#
# General body content is fleeced of what Flash can't read (a lot) and stuffed into
# a CDATA tag, thus CDATA content from the original html is ditched
# img tags are separated for data to be fed to the portfolio's image carousel
# object tags are replaced by a link back to the html
# applet tags are replaced by a javascript applet builder

use strict;
use LWP::Simple '!head';
use CGI ':standard';
use HTML::Parser;

# import requested html file
my $path =  'http://www.robotacid.com/assets/about.html';
if(param()){
  $path = param('url');
}
my $html = get ($path) or die ('Could not open $path : $!');

# Globals
my $folder = substr($path, 0, rindex($path, '/')).'/';
my @imgs = ();
my $xml = '';
my %applet = ();
my @paramName = ();
my @paramValue = ();

# Unacceptable tags regex $notag
my @notag = ("p", "div", "table", "tr", "td", "hr", "h1", "center", "acronym", "script");
my $notag = "^";
foreach(@notag){
  $notag .= $_.'$|^';
}
$notag = substr($notag, 0, length($notag)-2);

# node exorcism: CDATA
while($html =~ /<!\[CDATA\[|\]\]>/){
  my $start = index($html, "<![CDATA[");
  my $end = index($html, "]]>", $start) + 3;
  my $replacement = '';
  substr($html, $start, $end - $start, $replacement);
}

# node exorcism: Swap object tags with a link to html page
while($html =~ /<object/i){
  my $start = index(lc($html), "<object");
  my $end = index(lc($html), "/object>", $start) + 8;
  my $replacement = '<a href="'.$path.'" target="_blank">Object detected, click here to visit original page</a>';
  substr($html, $start, $end - $start, $replacement);
}

# module based Java Applet parsing
package ScanApplet;
use base "HTML::Parser";
sub start {
  my ($self, $tag, $attr, $attrseq, $origtext) = @_;
  if($tag =~ /^applet$/){
    if(defined $attr->{'code'}){
      $applet{'code'} = $attr->{'code'};
    }
    if(defined $attr->{'archive'}){
      $applet{'archive'} = '';
      if($attr->{'archive'} !~ /^http:/i){
        if($attr->{'archive'} =~ /,/){
          # pesky library files to check for full url
          my @files = split(/,/, $attr->{'archive'});
          foreach(@files){
            if($_ !~ /http:/i){
              $_  = $folder.$_;
            }
          }
          $applet{'archive'} = join(',', @files);
        } else {
          $applet{'archive'} = $folder.$attr->{'archive'};
        }
      } else {
        $applet{'archive'} = $attr->{'archive'};
      }
    }
    if(defined $attr->{'width'}){
      $applet{'width'} = $attr->{'width'};
    }
    if(defined $attr->{'height'}){
      $applet{'height'} = $attr->{'height'};
    }
    if(defined $attr->{'mayscript'}){
      $applet{'mayscript'} = $attr->{'mayscript'};
    }
  }
  if($tag =~ /^param$/){
    if(defined $attr->{'name'}){
      push(@paramName, $attr->{'name'});
    }
    if(defined $attr->{'value'}){
      push(@paramValue, $attr->{'value'});
    }
  }
}
sub text {
  my ($self, $text) = @_;
  $applet{'alt'} .= $text;
}

# Extract Applets and replace with uber JavaScript link to mount applet on page
# Unfortunately I can't get links with a lot of characters to activate from Flash ->
# The workaround is to sod the params and use a function in <head> to most stuff
while($html =~ /<applet/i){
  my $start = index(lc($html), "<applet");
  my $end = index(lc($html), "/applet>", $start) + 8;
  my $appletChunk =  substr($html, $start, $end - $start);
  # scan that shit
  my $a = new ScanApplet;
  $a->parse($appletChunk);
  $a->eof;
  # build that shit
  my $replacement = "<a href=\"javascript:setApplet('appletborder','";
  $replacement .= $applet{'width'}."','".$applet{'height'}."');";
  $replacement .= "var ao=new AppletObject('".$applet{'code'};
  $replacement .= "','".$applet{'archive'}."','".$applet{'width'}."','".$applet{'height'};
  if(defined $applet{'mayscript'}){
    $replacement .= "','".$applet{'mayscript'}."');";
  } else {
    $replacement .= "','true');";
  }
  #if(defined $applet{'alt'}){
  #  $replacement .= "ao.alt='".$applet{'alt'}."';";
  #}
  # param cobbler
  #if(@paramName > 0){
  #  for(my $i = 0; $i < @paramName; $i++){
  #    $replacement .= "ao.addParam('".$paramName[$i]."','".$paramValue[$i]."');";
  #  }
  #}
  #$replacement .= "ao.write('appletcontent');\">Java Applet detected, click here to mount on this page</a>";

  $replacement = "<a href=\"javascript:mountApplet('".$applet{'code'}."','".$applet{'archive'}."','".$applet{'width'}."','".$applet{'height'};
  if(defined $applet{'mayscript'}){
    $replacement .= "','".$applet{'mayscript'}."');";
  } else {
    $replacement .= "','true');";
  }
  $replacement .= "\">Java Applet detected, click here to mount on this page</a>";
  substr($html, $start, $end - $start, $replacement);
}

# Module based parsing - takes care of the more tricky entities
package ChangeEntities;
use base "HTML::Parser";
sub start {
  my ($self, $tag, $attr, $attrseq, $origtext) = @_;
  # kill img tags
  if($tag =~ /^img$|^myimg$/){
    if(defined $attr->{'src'}){
      my $ext = substr($attr->{'src'}, rindex($attr->{'src'}, '.'));
      if($ext =~ /\.jpg/i){
        if($attr->{'src'} !~ /^http:/){
          push(@imgs, $folder.$attr->{'src'});
        } else {
          push(@imgs, $attr->{'src'});
        }
      }
    }
    return;
  }
  # fix anchors
  if($tag =~ /^a$/){
    my $ext = '.html';
    if(defined $attr->{'href'}){
      $ext = substr($attr->{'href'}, rindex($attr->{'href'}, '.'));
      if($attr->{'href'} !~ /^http:\/\/|^mailto:|^javascript:/){
        $attr->{'href'} = $folder.$attr->{'href'};
      }
    }
    my $target =  ' target="_blank"';
    if($ext =~ /\.zip|\.doc/i || $attr->{'href'} =~ /^mailto:/ || $attr->{'href'} =~ /^javascript:/){
      $target = '';
    }
    $xml .= '<a href="'.$attr->{'href'}.'"'.$target.'>';
    return;
  }
  # remove attributes from body tag
  if($tag =~ /^body$/){
    $xml .= "<body>";
    return;
  }
  # replace unwanted tags
  if($tag =~ /$notag/){
    $xml .= '<br>';
    return;
  }
  $xml .= $origtext;
}
sub declaration{
}
sub text {
  my ($self, $text) = @_;
  $xml .= $text;
}
sub comment {
}
sub end {
  my ($self, $tag, $origtext) = @_;
  # replace unwanted tags
  if($tag =~ /$notag/){
    $xml .= '<br>';
    return;
  }
  $xml .= $origtext;
}

my $p = new ChangeEntities;
$p->parse($html);
$p->eof;
$html = $xml;

# change all whitespace to " "
$html =~ s/\s+/ /g;

# change &nbsp; to whitespace
$html =~ s/&nbsp;/ /g;

# separate consecutive anchors with commas (Flash htmlText bug)
$html =~ s/\/a>\s*<a/\/a>, <a/gi;

# put in link to original page
my $link = '<br><br><a href="'.$path.'" target="_blank">Visit HTML version of this url</a></body>';
$html =~ s/<\/body>/$link/i;

# make anchor tags visible in Flash - DEPRECATED, using external style sheet now
#$html =~ s/<a/<u><font color="#0066CC"><a/gi;
#$html =~ s/\/a>/\/a><\/font><\/u>/gi;

# remove more than 2 consecutive <br>
while($html =~ /<br>\s*<br>\s*<br>/gi){
  $html =~ s/<br>\s*<br>\s*<br>/<br><br>/gi;
}

# chomp whitespace on rightside of <br> tags
while($html =~ /<br>\s/gi){
  $html =~ s/<br>\s/<br>/gi
}

# isolate <body> node
my $start = index(uc($html), "<BODY>");
my $end = index(uc($html), "</BODY>") + 7;
my $body = substr($html, $start, $end - $start);

# Make the first line a title span
$body =~ s/>(\s*([^><\s]+\s*)+)</><span class="title">$1<\/span></;

# Force a <br> after <body>
$body =~ s/<body>/<body><br>/i;

# disolve multiple <br> after <body>
$body =~ s/(\s*<br>\s*)+/<br>/i;

# convert <body> tags to CDATA tags
$body =~ s/<body>/<![CDATA[/i;
$body =~ s/<\/body>/]]>/i;

# concat data, print result
$xml = '<root><htmltext>'.$body.'</htmltext><carousel>';
foreach(@imgs){
  $xml .= '<img src="'.$_.'" />';
}
$xml .= '</carousel></root>';
print "Content-type: text/xml\n\n", $xml;
